import {
  AlertService, Form, LoadingService, Modal, ModalService, Select,
} from '@zeedhi/common';
import { Http, IEventParam, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { ZdSelect } from '@zeedhi/vuetify';

export class Opl00800MigracaoRupturaOPLController {
  private alteracaoAutomaticaEntregaModal!: Modal;

  private confirmEDNewModal!: Modal;

  private confirmEDPaPModal!: Modal;

  public confirmDesfazerModal!: Modal;

  public confirmConclusaoModal!: Modal;

  public alteracaoAutomaticaProdutoModal!: Modal;

  public onChangeCdfiliopelog({ component }: IEventParam<any>) {
    const fieldRuptura = component.parent.getChildInstance('NRSEQRUPTURAOL');

    fieldRuptura.datasource.filter.P_CDFILIOPELOG = component.value.CDFILIAL;

    fieldRuptura.datasource.get();
  }

  public confirmarFiltro() {
    const form = Metadata.getInstance<Form>('fltroMigracaoRupturaOPL');
    const valid = form.validate();

    if (!valid) {
      AlertService.show({
        top: true,
        right: true,
        timeout: 2000,
        name: 'filtroInvalidoAltert',
        text: 'Preencha todos os campos obrigatórios!',
        color: 'error',
      });
    } else {
      form.submit();

      this.loadGrid();
    }
  }

  public loadGrid() {
    const grid = Metadata.getInstance<TekGrid>('gridMigracaoRupturaOPL');
    const fieldCDFILIOPERLOG = Metadata.getInstance<Select>('CDFILIOPELOG');
    const fieldNRSEQRUPTURAOL = Metadata.getInstance<Select>('NRSEQRUPTURAOL');
    const fieldCDPRODINI = Metadata.getInstance<Select>('CDPRODINI');
    const fieldCDPRODFIN = Metadata.getInstance<Select>('CDPRODFIN');

    const newFilter = {
      P_CDFILIOPERLOG: fieldCDFILIOPERLOG.value.CDFILIAL,
      P_NRSEQRUPTURAOL: fieldNRSEQRUPTURAOL.value.NRSEQRUPTURAOL,
      P_CDPRODINI: fieldCDPRODINI.value != null ? fieldCDPRODINI.value.CDARVPROD : null,
      P_CDPRODFIN: fieldCDPRODFIN.value != null ? (String(fieldCDPRODFIN.value.CDARVPROD).concat(String('ZZZZZZZZZZZZZZZZZZZZ'))).substring(0, 20) : null,
    };

    grid.datasource.filter = newFilter;
    grid.datasource.get().then((success) => {
      if (success.length > 0) {
        for (let i = 0; i < success.length; i += 1) {
          success[i].ID = i;
        }
      }
      grid.datasource.data = success;
    });

    this.createModals();
  }

  public async createModals() {
    this.alteracaoAutomaticaEntregaModal = ModalService.create({
      name: 'alteracaoAutomaticaEntregaModal',
      persistent: true,
      children: [
        {
          name: 'alteracaoAutomaticaEntregaHeader',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'titleText',
              component: 'ZdText',
              text: 'Alteração Automática de Data de Entrega',
              cssClass: 'zd-theme-font-title',
            },
          ],
          rightSlot: [
            {
              name: 'closeModalButton',
              component: 'ZdModalCloseButton',
              small: true,
              modalName: 'alteracaoAutomaticaEntregaModal',
            },
          ],
        },
        {
          name: 'form',
          component: 'ZdForm',
          children: [
            {
              name: 'datePicker',
              component: 'ZdDate',
              label: 'Data de Entrega',
              dateFormat: 'DD/MM/YYYY',
              inputFormat: 'DD/MM/YYYY',
              displayFormat: 'DD/MM/YYYY',
              validations: {
                required: {},
              },
            },
          ],
        },
        {
          name: 'alteracaoAutomaticaEntregaFooter',
          component: 'ZdFooter',
          color: 'transparent',
          padless: true,
          rightSlot: [
            {
              name: 'saveButton',
              component: 'ZdButton',
              label: 'Salvar',
              events: {
                click: '{{Opl00800MigracaoRupturaOPLController.alteracaoAutomaticaEntrega}}',
              },
            },
          ],
        },
      ],
    });
  }

  public destroyAlteracaoAutomaticaEntregaModal() {
    this.alteracaoAutomaticaEntregaModal?.destroy();
  }

  public openAlteracaoAutomaticaEntregaModal() {
    const grid = Metadata.getInstance<TekGrid>('gridMigracaoRupturaOPL');
    const rows = grid.selectedRows;

    if (rows.length > 0) {
      this.alteracaoAutomaticaEntregaModal.show();
    }
  }

  public closeAlteracaoAutomaticaEntregaModal() {
    this.alteracaoAutomaticaEntregaModal.hide();
    this.destroyAlteracaoAutomaticaEntregaModal();
  }

  public async alteracaoAutomaticaEntrega() {
    const grid = Metadata.getInstance<TekGrid>('gridMigracaoRupturaOPL');
    const rows = grid.selectedRows;
    const form = Metadata.getInstance<Form>('form');
    const valid = form.validate();

    if (rows.length > 0) {
      if (!valid) {
        AlertService.show({
          top: true,
          right: true,
          timeout: 2000,
          name: 'filtroInvalidoAltert',
          text: 'Preencha todos os campos obrigatórios!',
          color: 'error',
        });
      } else {
        form.submit();

        const row = {
          rowsUse: rows,
          P_DTPROGENTAF: form.value.datePicker,
        };

        await Http.post('/alteracaoAutomaticaEntrega', { requestType: 'Row', row }).then((response) => {
          this.closeAlteracaoAutomaticaEntregaModal();

          grid.reload();

          grid.datasource.get().then((success) => {
            if (success.length > 0) {
              for (let i = 0; i < success.length; i += 1) {
                success[i].ID = i;
              }
            }
            grid.datasource.data = success;
          });

          grid.deleteRows();

          if (response.data.dataset.data[0] === 'success') {
            AlertService.show({
              top: true,
              right: true,
              timeout: 2000,
              name: 'successOperation',
              text: 'Operação realizada com sucesso!',
              color: 'success',
            });
          } else {
            AlertService.show({
              top: true,
              right: true,
              timeout: 2000,
              name: 'failedOperation',
              text: 'Falha na execução da operação!',
              color: 'error',
            });
          }
        });
      }
    } else {
      this.destroyAlteracaoAutomaticaEntregaModal();
    }
  }

  public alteracaoAutomatica() {
    let permiteOperacao = true;

    this.alteracaoAutomaticaProdutoModal = ModalService.create({
      name: 'alteracaoAutomaticaProdutoModal',
      events: {},
      title: 'Alteração Automática de Produto',
      persistent: true,
      children: [
        {
          name: 'alteracaoAutomaticaProdutoForm',
          component: 'ZdForm',
          children: [
            {
              name: 'CDPRODUTO',
              component: 'ZdSelect',
              label: 'Produto',
              autocomplete: true,
              returnObject: true,
              grid: {
                sm: 12,
              },
              dataText: [
                {
                  name: 'CDARVPROD',
                  mask: '#.##.##.###.##',
                },
                'NMPRODUTO',
              ],
              dataValue: 'CDARVPROD',
              value: '',
              datasource: {
                route: '/crudN/produto',
                type: 'rest',
                limit: 30,
                lazyLoad: true,
                order: [
                  'CDARVPROD.asc',
                ],
                filter: {
                  CDNVPRODUTO: 5,
                },
              },
              validations: {
                required: {},
              },
              events: {},
            },
            {
              name: 'CDMARCA',
              component: 'ZdSelect',
              label: 'Marca',
              autocomplete: true,
              returnObject: true,
              grid: {
                sm: 12,
              },
              dataText: [
                'CDMARCA',
                'NMMARCA',
              ],
              dataValue: 'CDMARCA',
              value: '',
              datasource: {
                route: '/crudN/marca',
                type: 'rest',
                limit: 100,
                lazyLoad: true,
                order: [
                  'CDMARCA.asc',
                ],
                filter: {},
              },
              events: {},
            },
            {
              name: 'justificativa',
              label: 'Justificativa',
              component: 'ZdTextInput',
              grid: {
                sm: 12,
              },
              validations: {
                required: {},
              },
            },
          ],
        },
        {
          name: 'alteracaoAutomaticaProdutoFooter',
          component: 'ZdFooter',
          color: 'transparent',
          padless: true,
          leftSlot: [
            {
              name: 'cancelAlteracaoAutomaticaProdutoButton',
              component: 'ZdButton',
              label: 'Cancelar',
              events: {
                click: '{{Opl00800MigracaoRupturaOPLController.closeAlteracaoAutomaticaProdutoModal}}',
              },
            },
          ],
          rightSlot: [
            {
              name: 'alteracaoAutomaticaProdutoButton',
              component: 'ZdButton',
              label: 'Prosseguir',
              events: {
                click: '{{Opl00800MigracaoRupturaOPLController.confirmAlteracaoAutomaticaProduto}}',
              },
            },
          ],
        },
      ],
    });

    const grid = Metadata.getInstance<TekGrid>('gridMigracaoRupturaOPL');
    const rows = grid.selectedRows;
    const rowBase = rows[0];

    for (let i = 0; i < rows.length; i += 1) {
      if (rows[i].CDARVPROD !== rowBase.CDARVPROD) {
        permiteOperacao = false;
        break;
      }
    }

    if (permiteOperacao) {
      if (rows.length > 0) {
        this.alteracaoAutomaticaProdutoModal.show();
      } else {
        this.alteracaoAutomaticaProdutoModal?.destroy();
      }
    } else {
      AlertService.show({
        top: true,
        right: true,
        timeout: 2000,
        name: 'produtosDiferentes',
        text: 'Para realizar essa operação, os itens selecionados devem possuir o mesmo produto.',
        color: 'error',
      });
    }
  }

  public closeAlteracaoAutomaticaProdutoModal() {
    this.alteracaoAutomaticaProdutoModal.hide();
    this.alteracaoAutomaticaProdutoModal?.destroy();
  }

  public async confirmAlteracaoAutomaticaProduto() {
    const grid = Metadata.getInstance<TekGrid>('gridMigracaoRupturaOPL');
    const rows = grid.selectedRows;
    const form = Metadata.getInstance<Form>('alteracaoAutomaticaProdutoForm');
    const formFiltro = Metadata.getInstance<Form>('fltroMigracaoRupturaOPL');
    const valid = form.validate();
    const fieldCDPRODUTO = Metadata.getInstance<Select>('CDPRODUTO');
    const fieldCDMARCA = Metadata.getInstance<Select>('CDMARCA');
    const prodIniField = formFiltro.getChildInstance<ZdSelect>('CDPRODINI');
    const prodFinField = formFiltro.getChildInstance<ZdSelect>('CDPRODFIN');
    const fieldCDFILIOPERLOG = Metadata.getInstance<Select>('CDFILIOPELOG');
    const fieldNRSEQRUPTURAOL = Metadata.getInstance<Select>('NRSEQRUPTURAOL');
    const row = {
      P_CDFILIOPERLOG: fieldCDFILIOPERLOG.value.CDFILIAL,
      P_NRSEQRUPTURAOL: fieldNRSEQRUPTURAOL.value.NRSEQRUPTURAOL,
      CDPRODUTO: fieldCDPRODUTO.value.CDARVPROD,
      CDMARCA: fieldCDMARCA.value.CDMARCA,
      ROWS: rows,
      P_CDPRODINI: prodIniField.value !== null ? prodIniField.value.CDARVPROD : null,
      P_CDPRODFIN: prodFinField.value !== null ? prodFinField.value.CDARVPROD : null,
      P_MENS: form.value.justificativa,
    };

    if (!valid) {
      AlertService.show({
        top: true,
        right: true,
        timeout: 2000,
        name: 'filtroInvalidoAltert',
        text: 'Preencha todos os campos obrigatórios!',
        color: 'error',
      });
    } else {
      await Http.post('/alteracaoAutomaticaProduto', { requestType: 'Row', row }).then((response) => {
        if (response.data.length === 0) {
          AlertService.show({
            top: true,
            right: true,
            timeout: 2000,
            name: 'successOperation',
            text: 'Operação realizada com sucesso!',
            color: 'success',
          });
          this.closeAlteracaoAutomaticaProdutoModal();
        } else if (response.data.error) {
          AlertService.show({
            top: true,
            right: true,
            timeout: 6000,
            name: 'filtroInvalidoAltert',
            text: response.data.error,
            color: 'error',
          });
        }
      });
    }
  }

  public entregaDiretaNew() {
    this.confirmEDNewModal = ModalService.create({
      name: 'entregaDiretaNewModal',
      events: {},
      title: 'Deseja prosseguir com a Entrega Direta (Nova Apuração) para o(s) item(s) selecionado(s)?',
      persistent: true,
      children: [
        {
          name: 'PersistentModalFormED',
          component: 'ZdForm',
          children: [
            {
              name: 'closeModalEDButton',
              component: 'ZdButton',
              label: 'Cancelar',
              events: {
                click: '{{Opl00800MigracaoRupturaOPLController.closeConfirmEDNewModal}}',
              },
            },
            {
              name: 'confirmEDButton',
              component: 'ZdButton',
              label: 'Prosseguir',
              events: {
                click: '{{Opl00800MigracaoRupturaOPLController.confirmEntregaDiretaNew}}',
              },
            },
          ],
        },
      ],
    });

    const grid = Metadata.getInstance<TekGrid>('gridMigracaoRupturaOPL');
    const rows = grid.selectedRows;

    if (rows.length > 0) {
      this.confirmEDNewModal.show();
    } else {
      this.confirmEDNewModal?.destroy();
    }
  }

  public closeConfirmEDNewModal() {
    this.confirmEDNewModal.hide();
    this.confirmEDNewModal?.destroy();
  }

  public async confirmEntregaDiretaNew() {
    const grid = Metadata.getInstance<TekGrid>('gridMigracaoRupturaOPL');
    const rows = grid.selectedRows;

    await Http.post('/entregaDiretaNew', { requestType: 'Row', rows }).then((response) => {
      this.closeConfirmEDNewModal();

      grid.reload();

      grid.datasource.get().then((success) => {
        if (success.length > 0) {
          for (let i = 0; i < success.length; i += 1) {
            success[i].ID = i;
          }
        }
        grid.datasource.data = success;
      });

      grid.deleteRows();

      if (response.data.dataset.data[0] === 'success') {
        AlertService.show({
          top: true,
          right: true,
          timeout: 2000,
          name: 'successOperation',
          text: 'Operação realizada com sucesso!',
          color: 'success',
        });
      } else {
        AlertService.show({
          top: true,
          right: true,
          timeout: 2000,
          name: 'failedOperation',
          text: 'Houve uma falha na execução da operação!',
          color: 'error',
        });
      }
    });
  }

  public entregaDiretaPaP() {
    this.confirmEDPaPModal = ModalService.create({
      name: 'entregaDiretaPaPModal',
      events: {},
      title: 'Deseja prosseguir com a Entrega Direta (Ponto a Ponto) para o(s) item(s) selecionado(s)?',
      persistent: true,
      children: [
        {
          name: 'PersistentModalFormPaP',
          component: 'ZdForm',
          children: [
            {
              name: 'closeModalPaPButton',
              component: 'ZdButton',
              label: 'Cancelar',
              events: {
                click: '{{Opl00800MigracaoRupturaOPLController.closeConfirmEDPaPModal}}',
              },
            },
            {
              name: 'confirmPaPButton',
              component: 'ZdButton',
              label: 'Prosseguir',
              events: {
                click: '{{Opl00800MigracaoRupturaOPLController.confirmEntregaDiretaPaP}}',
              },
            },
          ],
        },
      ],
    });

    const grid = Metadata.getInstance<TekGrid>('gridMigracaoRupturaOPL');
    const rows = grid.selectedRows;

    if (rows.length > 0) {
      this.confirmEDPaPModal.show();
    } else {
      this.confirmEDPaPModal?.destroy();
    }
  }

  public closeConfirmEDPaPModal() {
    this.confirmEDPaPModal.hide();
    this.confirmEDPaPModal?.destroy();
  }

  public async confirmEntregaDiretaPaP() {
    const grid = Metadata.getInstance<TekGrid>('gridMigracaoRupturaOPL');
    const rows = grid.selectedRows;

    await Http.post('/entregaDiretaPaP', { requestType: 'Row', rows }).then((response) => {
      this.closeConfirmEDPaPModal();

      grid.reload();

      grid.datasource.get().then((success) => {
        if (success.length > 0) {
          for (let i = 0; i < success.length; i += 1) {
            success[i].ID = i;
          }
        }
        grid.datasource.data = success;
      });

      grid.deleteRows();

      if (response.data.dataset.data[0] === 'success') {
        AlertService.show({
          top: true,
          right: true,
          timeout: 2000,
          name: 'successOperation',
          text: 'Operação realizada com sucesso!',
          color: 'success',
        });
      } else {
        AlertService.show({
          top: true,
          right: true,
          timeout: 2000,
          name: 'failedOperation',
          text: 'Houve uma falha na execução da operação!',
          color: 'error',
        });
      }
    });
  }

  public desfazerMigracaoItem() {
    this.confirmDesfazerModal = ModalService.create({
      name: 'desfazerMigracaoItemModal',
      events: {},
      title: 'Deseja prosseguir para o(s) item(s) selecionado(s)?',
      persistent: true,
      children: [
        {
          name: 'desfazerMigracaoItemForm',
          component: 'ZdForm',
          children: [
            {
              name: 'closeModalDesfazerButton',
              component: 'ZdButton',
              label: 'Cancelar',
              events: {
                click: '{{Opl00800MigracaoRupturaOPLController.closeConfirmDesfazerModal}}',
              },
            },
            {
              name: 'confirmModalDesfazerButton',
              component: 'ZdButton',
              label: 'Prosseguir',
              events: {
                click: '{{Opl00800MigracaoRupturaOPLController.confirmDesfazerItem}}',
              },
            },
          ],
        },
      ],
    });

    const grid = Metadata.getInstance<TekGrid>('gridMigracaoRupturaOPL');
    const rows = grid.selectedRows;

    if (rows.length > 0) {
      this.confirmDesfazerModal.show();
    } else {
      this.confirmDesfazerModal?.destroy();
    }
  }

  public closeConfirmDesfazerModal() {
    this.confirmDesfazerModal.hide();
    this.confirmDesfazerModal?.destroy();
  }

  public async confirmDesfazerItem() {
    const grid = Metadata.getInstance<TekGrid>('gridMigracaoRupturaOPL');
    const rows = grid.selectedRows;

    await Http.post('/desfazerMigracaoItem', { requestType: 'Row', rows }).then((response) => {
      this.closeConfirmDesfazerModal();

      grid.reload();

      grid.datasource.get().then((success) => {
        if (success.length > 0) {
          for (let i = 0; i < success.length; i += 1) {
            success[i].ID = i;
          }
        }
        grid.datasource.data = success;
      });

      grid.deleteRows();

      if (response.data.dataset.data[0] === 'success') {
        AlertService.show({
          top: true,
          right: true,
          timeout: 2000,
          name: 'successOperation',
          text: 'Operação realizada com sucesso!',
          color: 'success',
        });
      } else {
        AlertService.show({
          top: true,
          right: true,
          timeout: 2000,
          name: 'failedOperation',
          text: 'Houve uma falha na execução da operação!',
          color: 'error',
        });
      }
    });
  }

  public concluirAnalise() {
    const form = Metadata.getInstance<Form>('fltroMigracaoRupturaOPL');
    const valid = form.validate();
    const prodIniField = form.getChildInstance<ZdSelect>('CDPRODINI');
    const prodFinField = form.getChildInstance<ZdSelect>('CDPRODFIN');

    if (prodIniField.value !== null && prodFinField.value !== null) {
      AlertService.show({
        top: true,
        right: true,
        timeout: 2000,
        name: 'prodIniAndProdFinFilled',
        text: 'É necessário desmarcar o Produto Inicial e Final no Filtro para prosseguir com esta operação!',
        color: 'error',
      });
    } else if (!valid) {
      AlertService.show({
        top: true,
        right: true,
        timeout: 2000,
        name: 'filtroInvalidoAltert',
        text: 'Preencha todos os campos obrigatórios!',
        color: 'error',
      });
    } else {
      this.confirmConclusaoModal = ModalService.create({
        name: 'concluirAnaliseModal',
        events: {},
        title: 'Deseja prosseguir com a Conlusão da Análise?',
        persistent: true,
        children: [
          {
            name: 'concluirAnaliseForm',
            component: 'ZdForm',
            children: [
              {
                name: 'justificativa',
                label: 'Justificativa',
                component: 'ZdTextInput',
                grid: {
                  sm: 12,
                },
                validations: {
                  required: {},
                },
              },
            ],
          },
          {
            name: 'concluirAnaliseFooter',
            component: 'ZdFooter',
            color: 'transparent',
            padless: true,
            leftSlot: [
              {
                name: 'cancelConcluirAnaliseButton',
                component: 'ZdButton',
                label: 'Cancelar',
                events: {
                  click: '{{Opl00800MigracaoRupturaOPLController.closeConfirmConcluirAnaliseModal}}',
                },
              },
            ],
            rightSlot: [
              {
                name: 'concluirAnaliseButton',
                component: 'ZdButton',
                label: 'Prosseguir',
                events: {
                  click: '{{Opl00800MigracaoRupturaOPLController.confirmConcluirAnalise}}',
                },
              },
            ],
          },
        ],
      });

      this.confirmConclusaoModal.show();
    }
  }

  public closeConfirmConcluirAnaliseModal() {
    this.confirmConclusaoModal.hide();
    this.confirmConclusaoModal?.destroy();
  }

  public async confirmConcluirAnalise() {
    const form = Metadata.getInstance<Form>('concluirAnaliseForm');
    const valid = form.validate();
    const fieldCDFILIOPERLOG = Metadata.getInstance<Select>('CDFILIOPELOG');
    const fieldNRSEQRUPTURAOL = Metadata.getInstance<Select>('NRSEQRUPTURAOL');

    const row = {
      P_CDFILIOPERLOG: fieldCDFILIOPERLOG.value.CDFILIAL,
      P_NRSEQRUPTURAOL: fieldNRSEQRUPTURAOL.value.NRSEQRUPTURAOL,
      P_CDPRODINI: null,
      P_CDPRODFIN: null,
      P_MENS: form.value.justificativa,
    };

    if (!valid) {
      AlertService.show({
        top: true,
        right: true,
        timeout: 2000,
        name: 'filtroInvalidoAltert',
        text: 'Preencha todos os campos obrigatórios!',
        color: 'error',
      });
    } else {
      await Http.post('/concluirAnaliseRuptura', { requestType: 'Row', row }).then((response) => {
        this.closeConfirmConcluirAnaliseModal();
  
        if (response.data.dataset.data.P_ERROS === 'N') {
          AlertService.show({
            top: true,
            right: true,
            timeout: 2000,
            name: 'successOperation',
            text: 'Operação realizada com sucesso!',
            color: 'success',
          });
        } else {
          AlertService.show({
            top: true,
            right: true,
            timeout: 2000,
            name: 'failedOperation',
            text: 'Houve uma falha na execução da operação!',
            color: 'error',
          });
        }
      });
    }
  }

  public async rowClick({ component }: any) {
    LoadingService.show('Carregando...');

    const fieldCDFILIOPERLOG = Metadata.getInstance<Select>('CDFILIOPELOG');
    const textESTOQDISPODT = Metadata.getInstance<Select>('textESTOQ_DISPO_DT');
    const textESTOQDISPO = Metadata.getInstance<Select>('textESTOQ_DISPO');
    const row = component.datasource.currentRow;

    row.P_CDFILIOPERLOG = fieldCDFILIOPERLOG.value.CDFILIAL;
    
    textESTOQDISPODT.value = String(component.datasource.currentRow.QTPROGENTAFITEM).concat(',000');

    await Http.post('/getEstoqueProduto', { requestType: 'Row', row }).then((response) => {
      if (response.data.dataset.estoque.ESTOQ_DISPO) {
        textESTOQDISPO.value = String(response.data.dataset.estoque.ESTOQ_DISPO).concat(',000');
        LoadingService.hide();
      } else {
        textESTOQDISPO.value = '0,000';
        LoadingService.hide();
      }
    });
  }
}
